/*  RTC_TOGGLE 
    uses real time clock to generate an interrupt and
    toggles parallel port on each irq
    Originally by Michael "FZ" Barabanov(c) 1997, Released under the GPL.
    Modified by Victor Yodaiken (c) 1998, Released under the GPL.
    */

#include <linux/module.h>
#include <linux/kernel.h>
#include <rtl_fifo.h>
#include <asm/rt_irq.h>
#include <asm/rt_time.h>
#include <rtl_sched.h>
#include <asm/io.h>
#include <linux/cons.h>
#include "common.h"
#include <rtl_sync.h>


static int output = 0xffff;
#include <linux/mc146818rtc.h>
char save_cmos_A;
char save_cmos_B;
void shutdown(void);
static int shutdown_flag = 0;

void handler(void) {
	/* Do not	rt_task_wait() in a handler */
	switch(shutdown_flag){
		case 0:
			outb(output, LPT_PORT);
			/* I'm too lazy to figure out which pin is which
			   so just toggle all of them */
			output = ~output;
			(void) CMOS_READ(RTC_REG_C);  /* clear IRQ */ 
			break;
		case 1: 
			shutdown_flag = 2;
		default: /* do nothing */
	}

}


int init_module(void)
{
	unsigned char ctemp;
	int old_irq_state;
        printk("Starting parallel port module\n");
	rtl_no_interrupts(old_irq_state);
	request_RTirq(8, handler);
	/* program the RTC to interrupt at 8192 Hz */
	save_cmos_A = CMOS_READ(RTC_REG_A);
	save_cmos_B = CMOS_READ(RTC_REG_B);

	CMOS_WRITE(0x23, RTC_REG_A);  	/* 32kHz Time Base, 8192 Hz interrupt frequency */
	ctemp = CMOS_READ(RTC_REG_B);
	ctemp &= 0x8f;                	/* Clear */
	ctemp |= 0x40;                	/* Periodic interrupt enable */
	CMOS_WRITE(ctemp, RTC_REG_B); 

	(void) CMOS_READ(RTC_REG_C);
	rtl_restore_interrupts(old_irq_state);
	return 0;
}

void shutdown(){
	int old_irq_state;
	rtl_no_interrupts(old_irq_state);
	CMOS_WRITE(save_cmos_A, RTC_REG_A);
	CMOS_WRITE(save_cmos_B, RTC_REG_B);
	free_RTirq(8);
	rtl_restore_interrupts(old_irq_state);
}


void cleanup_module(void)
{
	int timeout = 100000;
	shutdown_flag = 1;
	while((shutdown_flag==1) && timeout--);
	/* so it's kinda sloppy, wait for timeout or for 
	   interrupt routine to ack shutdown, whatever comes
	   first */
	shutdown();
}
