#ifndef __RT__SCHED__
#define __RT__SCHED__

#include <asm/rt_time.h>
#include "../schedulers/x86/fpu.h"

#define RT_TASK_MAGIC 0x754d2774

#define RT_LOWEST_PRIORITY 1000000


enum {
	RT_TASK_READY, RT_TASK_DELAYED, RT_TASK_DORMANT, RT_TASK_ACTIVE, RT_TASK_ZOMBIE
};

struct rt_task_struct {
	int *stack;	/* hardcoded */
	int uses_fp;	/* this one is too   */
	int magic;
	int state;
	int *stack_bottom;
	int priority;
	RTIME period;
	RTIME resume_time;
	struct rt_task_struct *next;
	RTL_FPU_CONTEXT fpu_regs;
};

typedef struct rt_task_struct RT_TASK;

#include "../schedulers/x86/switch.h"

extern RT_TASK *rtl_current;
extern RT_TASK rtl_linux_task;

#define rtl_is_linux_task(a) (a == &rtl_linux_task)


extern int rt_task_init(RT_TASK *task, void (*fn)(int data), int data, int stack_size,  int priority);


#define rtl_init (a,b,c,d,e) { rt_task_init(a,b,c,d,e); }
#define rtl_fpinit (a,b,c,d,e) { rt_task_init(a,b,c,d,e); rt_task_use_fp (a, 1); }

/* extern int rt_task_set_priority (RT_TASK *task, int priority); */

extern int rt_task_use_fp (RT_TASK *task, int flag);

extern int rt_task_delete (RT_TASK *task);

extern int rt_task_wait (void);

/* extern int rt_task_wait_until (RT_TASK *task, RTIME t); */

extern int rt_task_suspend (RT_TASK *task);

extern int rt_task_wakeup (RT_TASK *task);

extern int rt_task_make_periodic(RT_TASK *task, RTIME start_time, RTIME period);



/* these two set the _scheduler_ mode */
extern RTIME rtl_set_periodic_mode (RTIME period);
extern int rtl_set_oneshot_mode (void);


#endif
